#include "mpr121.h"
#include <Wire.h>
#include <MIDI.h>

int irqpin = A2;  // Digital 2
boolean touchStates[12]; //to keep track of the previous touch states

boolean isDragging, isScrolling;
#define DIRECTIONS 8
float dirVelo[DIRECTIONS];

float mouseVelFactor = 0.1; // 0.1 was a good speed for Lyn, 0.3 good for me - NO MPR CHIP
int initSpeed = 20;
float accel = 0.8; // controls the acceleration of the mouse

float scrollFactor = 1.0; //HERE

float point7 = 1.0 / sqrt(2.0);
float xMag[] = {
  -point7, -1, -point7, 0, point7, 1, point7, 0};
float yMag[] = {
  -point7, 0, point7, 1, point7, 0, -point7, -1};

void setup(){
  pinMode(irqpin, INPUT);
  digitalWrite(irqpin, HIGH); //enable pullup resistor

  MIDI.begin(4);
  Serial.begin(115200);
  delay(1500); // so there's time to upload a new sketch when arduino first plugged in,
  // before it hyjacks the mouse control.
  for (int i = 0; i < DIRECTIONS; i++) {
    dirVelo[i] = initSpeed;
  }  
  Wire.begin();

  mpr121_setup();
  isDragging = isScrolling = false;
}

void loop(){
  readTouchInputs();
  
  // Delay needed because the MPR runs very fast
  // and because the minimum Mouse.move is 1.0
  // These combine to make the slowest mouse speed way too quick without slowing down the Arduino.
  delay(13);
  
  for ( int i=0; i < 8; i++){
    if (touchStates[i+2]){
      if(!isScrolling){
        dirVelo[i] += accel;
        float xMove = mouseVelFactor * dirVelo[i] * xMag[i];
        float yMove = mouseVelFactor * dirVelo[i] * yMag[i];
        // xMove and yMove must be > 1, in order to register any move at all.
        Mouse.move(xMove, yMove, 0);
      } 
      else {
        // we are scrolling
        scrollFactor = scrollFactor + 0.1;
        Mouse.move(0, 0, 2 * yMag[i] * scrollFactor); //HERE
        delay(80);
      }
    }
  }
}


void readTouchInputs(){
  if(!checkInterrupt()){

    //read the touch state from the MPR121
    Wire.requestFrom(0x5A,2); 

    byte LSB = Wire.read();
    byte MSB = Wire.read();

    uint16_t touched = ((MSB << 8) | LSB); //16bits that make up the touch states


    for (int i=0; i < 12; i++){  // Check what electrodes were pressed
      if(touched & (1<<i)){
        // SAM's code here
        // think about converting this into a function
        if(i < 10 && i >= 2){
          // Do nothing
          // this only triggers upon interrupt, not when held down
        }
        else {
          // left or right click, or drag / scroll
          if(touchStates[i] == 0){
            //pin i was just touched
            //this used to debounce the mouse clicks, drag and scroll buttons
            if (i==0)
              Mouse.press(MOUSE_LEFT);
            else if (i==1)
              Mouse.press(MOUSE_RIGHT);
            else if (i==11) {
              isDragging = !isDragging;
              if (isDragging)
                Mouse.press(MOUSE_LEFT);
              else
                Mouse.release(MOUSE_LEFT);
            }
            else if (i==10)
              isScrolling = !isScrolling;
          } //else if(touchStates[i] == 1) {
          ////pin i is still being touched
          //// do nothing when click is held down, 
          //}
        }
        // END sam's code
        touchStates[i] = 1;      
      }
      else{
        if(touchStates[i] == 1){
          //pin i is no longer being touched
          // reset the velocity of UP direction
          if(i < 10 && i >= 2) {
            dirVelo[i-2] = initSpeed;
            scrollFactor = 1.0; //HERE
          }
          else {
            // release the mouse click buttons
            if (i == 0)
              Mouse.release(MOUSE_LEFT);
            else if (i == 1)
              Mouse.release(MOUSE_RIGHT);
          }
        }

        touchStates[i] = 0;
      }
    }
  }
}




void mpr121_setup(void){

  set_register(0x5A, ELE_CFG, 0x00); 

  // Section A - Controls filtering when data is > baseline.
  set_register(0x5A, MHD_R, 0x01);
  set_register(0x5A, NHD_R, 0x01);
  set_register(0x5A, NCL_R, 0x00);
  set_register(0x5A, FDL_R, 0x00);

  // Section B - Controls filtering when data is < baseline.
  set_register(0x5A, MHD_F, 0x01);
  set_register(0x5A, NHD_F, 0x01);
  set_register(0x5A, NCL_F, 0xFF);
  set_register(0x5A, FDL_F, 0x02);

  // Section C - Sets touch and release thresholds for each electrode
  set_register(0x5A, ELE0_T, TOU_THRESH);
  set_register(0x5A, ELE0_R, REL_THRESH);

  set_register(0x5A, ELE1_T, TOU_THRESH);
  set_register(0x5A, ELE1_R, REL_THRESH);

  set_register(0x5A, ELE2_T, TOU_THRESH);
  set_register(0x5A, ELE2_R, REL_THRESH);

  set_register(0x5A, ELE3_T, TOU_THRESH);
  set_register(0x5A, ELE3_R, REL_THRESH);

  set_register(0x5A, ELE4_T, TOU_THRESH);
  set_register(0x5A, ELE4_R, REL_THRESH);

  set_register(0x5A, ELE5_T, TOU_THRESH);
  set_register(0x5A, ELE5_R, REL_THRESH);

  set_register(0x5A, ELE6_T, TOU_THRESH);
  set_register(0x5A, ELE6_R, REL_THRESH);

  set_register(0x5A, ELE7_T, TOU_THRESH);
  set_register(0x5A, ELE7_R, REL_THRESH);

  set_register(0x5A, ELE8_T, TOU_THRESH);
  set_register(0x5A, ELE8_R, REL_THRESH);

  set_register(0x5A, ELE9_T, TOU_THRESH);
  set_register(0x5A, ELE9_R, REL_THRESH);

  set_register(0x5A, ELE10_T, TOU_THRESH);
  set_register(0x5A, ELE10_R, REL_THRESH);

  set_register(0x5A, ELE11_T, TOU_THRESH);
  set_register(0x5A, ELE11_R, REL_THRESH);

  // Section D
  // Set the Filter Configuration
  // Set ESI2
  set_register(0x5A, FIL_CFG, 0x04);

  // Section E
  // Electrode Configuration
  // Set ELE_CFG to 0x00 to return to standby mode
  set_register(0x5A, ELE_CFG, 0x0C);  // Enables all 12 Electrodes


  // Section F
  // Enable Auto Config and auto Reconfig
  /*set_register(0x5A, ATO_CFG0, 0x0B);
   set_register(0x5A, ATO_CFGU, 0xC9);  // USL = (Vdd-0.7)/vdd*256 = 0xC9 @3.3V   set_register(0x5A, ATO_CFGL, 0x82);  // LSL = 0.65*USL = 0x82 @3.3V
   set_register(0x5A, ATO_CFGT, 0xB5);*/  // Target = 0.9*USL = 0xB5 @3.3V

  set_register(0x5A, ELE_CFG, 0x0C);

}


boolean checkInterrupt(void){
  return digitalRead(irqpin);
}


void set_register(int address, unsigned char r, unsigned char v){
  Wire.beginTransmission(address);
  Wire.write(r);
  Wire.write(v);
  Wire.endTransmission();
}



